/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.dsl

import ch.qos.logback.classic.Level
import ch.qos.logback.classic.Logger
import metridoc.extension.ExtensionRunner
import metridoc.utils.PropertyUtils
import org.slf4j.LoggerFactory

/**
 * Primary interface to run jobs.  Useful for embedding metridoc into standard java apps as well.  For
 * {@link JobBuilder} documentation see http://code.google.com/p/metridoc/wiki/JobBuilder, for documentation on job
 * files please see http://code.google.com/p/metridoc/wiki/JobFile
 *
 *
 */
class JobBuilder {

    def ExtensionRunner extensionRunner
    def Binding services = new Binding()
    def JobManager manager
    def log = LoggerFactory.getLogger("${METRIDOC_JOBS}.builder")
    boolean stopCalled = false
    private static Set markedJobs = []

    public static final String DEFAULT_JOB = "defaultJob"
    public static final String METRIDOC_JOBS = "metridoc.jobs"
    public static final String METRIDOC_BUILDER = "${METRIDOC_JOBS}.builder"

    ExtensionRunner getExtensionRunner() {
        if (extensionRunner) {
            return extensionRunner
        }

        extensionRunner = new ExtensionRunner(jobManager: getJobManager())
    }

    def JobBuilder load(String... jobFiles) {
        getJobManager().parseJobs jobFiles
        return this
    }

    def JobBuilder load(Class... jobClasses) {
        log.debug("loading jobs ${jobClasses}")
        getJobManager().parseJobs jobClasses
        return this
    }

    def JobManager getJobManager() {
        if (manager == null) {
            manager = new JobManager(services: services)
        }

        return manager
    }

    def void run() {
        log.debug("no job specified, so '${DEFAULT_JOB}' will be run")
        run(new LinkedHashMap(), DEFAULT_JOB)
    }

    def void run(LinkedHashMap args = null, String... jobs) {
        configureBuilder this, args
        getJobManager().runJobs jobs
    }

    private static void setLoggingLevel(Level logLevel) {
        Logger logger = (Logger) LoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME)
        logger.setLevel(logLevel)
    }

    def static void run(LinkedHashMap args = null, Closure closure) {
        run(args, DEFAULT_JOB, closure)
    }

    def static void run(LinkedHashMap args = null, String jobName, Closure closure) {
        JobBuilder.run(args, [jobName] as String[], closure)
    }

    def static void run(LinkedHashMap args = null, String[] jobNames, Closure closure) {
        def builder = new JobBuilder()
        try {
            configureBuilder builder, args
            def jobManager = builder.getJobManager()
            if (closure) {
                jobManager.loadJobs closure
            }
            builder.addPlugins(JobRunner)
            jobManager.runJobs jobNames
        } finally {
            builder.stop()
        }
    }

    def static void jobs(LinkedHashMap args = null, Closure closure) {
        jobs(args, DEFAULT_JOB, closure)
    }

    def static void jobs(LinkedHashMap args = null, String jobName, Closure closure) {
        run(args, jobName, closure)
    }

    def static isJob(Object job) {
        if (markedJobs.contains(job)) return; //already declared it as a job
        markedJobs.add(job)

        def builder = getDefaultJobBuilder()
        def manager = builder.getJobManager()
        job.metaClass.mixin JobRunner
        def jobRunner = job as JobRunner
        jobRunner.setServices(manager.services)
        if (job instanceof Script) {
            jobRunner.setServices(job.binding)
            job.out = System.out
        }

        def plugins = builder.getExtensionRunner().plugins
        if (plugins) {
            plugins.each {plugin ->
                job.metaClass.mixin(plugin)
            }
        }
        jobRunner.setPlugins(plugins)

        return job
    }

    def addPlugins(owner) {
        def plugins = getExtensionRunner().getPlugins()

        plugins.each {
            owner.mixin it
        }
    }

    static JobBuilder getDefaultJobBuilder() {
        def defaultJobBuilder = new JobBuilder()
        configureBuilder(defaultJobBuilder, null)
        return defaultJobBuilder
    }

    static JobRunner job(Binding binding = null) {
        def builder = getDefaultJobBuilder()
        def jobRunner = new JobRunner()
        jobRunner.services = builder.manager.services

        if (binding) {
            binding.variables.each {key, value ->
                jobRunner.services.setVariable(key, value)
            }
        }

        def plugins = builder.getExtensionRunner().plugins
        if (plugins) {
            plugins.each {plugin ->
                jobRunner.metaClass.mixin(plugin)
            }
        }

        return jobRunner
    }

    def void stop() {
        extensionRunner.onStop.each {Closure onStop ->
            onStop.call(this)
        }
        stopCalled = true
    }

    private static void configureBuilder(JobBuilder builder, args) {
        builder.extensionRunner.binding.setVariable("jobBuilder", builder)
        builder.extensionRunner.runExtensions()
        builder.extensionRunner.jobBuilderConfigurations.each {Closure config ->
            config.call(args, builder)
        }

        Runtime.runtime.addShutdownHook(
                new Thread({
                    if (!builder.stopCalled) {
                        builder.stop()
                    }
                })
        )
    }

    private validateConfig(config) {
        ArtifactType.values().each {artifact ->
            artifact.validate(config)
        }
    }
}
