/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package metridoc.dsl

import metridoc.utils.Assert
import org.slf4j.LoggerFactory

/**
 *
 * @author tbarker
 */
class JobManager {

    def JobRunner jobRunner
    def Binding services = new Binding()
    def ClassLoader jobClassLoader = Thread.currentThread().contextClassLoader
    def log = LoggerFactory.getLogger("${JobBuilder.METRIDOC_JOBS}.manager")
    def String metridocDirectory = System.getProperty("user.home") + "/.metridoc"
    def boolean firstPassComplete = false
    def boolean atLeastOneJobIsLoaded = false

    def void parseJobs(String... jobFiles) {
        jobFiles.each {

            def fileName = it
            if (!fileName.endsWith(".groovy")) {
                fileName += ".groovy"
            }

            def stream = jobClassLoader.getResourceAsStream(fileName)
            def jobsContainer
            if (stream != null) {
                def reader = new InputStreamReader(jobClassLoader.getResourceAsStream(fileName))
                def shell = new GroovyShell()
                jobsContainer = shell.parse(reader, fileName)
            } else {
                try {
                    def clazz = jobClassLoader.loadClass(it)
                    jobsContainer = clazz.newInstance()
                } catch (ClassNotFoundException ex) {
                    throw new IllegalArgumentException("Could not find job ${it} as a file or class")
                }
            }

            runContainer jobsContainer
        }

        firstPassComplete = true
    }

    def void parseJobs(Class... jobClasses) {
        log.debug "parsing job(s) ${jobClasses}"
        jobClasses.each {clazz ->
            def jobsContainer = clazz.newInstance()
            runContainer jobsContainer
        }
    }

    private void runContainer(jobsContainer) {
        log.debug "loading jobs method into ${jobsContainer}"
        jobsContainer.getMetaClass().jobs {
            this.loadJobs(it, false)
        }

        log.debug "loading importJobs method into ${jobsContainer}"
        jobsContainer.metaClass.importJobs = {Class[] jobs ->
            importJobs(jobs)
        }

        log.debug "running jobs container ${jobsContainer}"
        jobsContainer.run()
    }

    def void importJobs(Class... jobs) {
        parseJobs(jobs)
    }

    void loadJobs(Closure closure, firstPassComplete = true) {
        closure.setDelegate(this)
        closure.setDirective(Closure.DELEGATE_ONLY)
        closure()
        this.firstPassComplete = firstPassComplete
    }

    void job(String name, Closure closure) {
        job name, null, closure
    }

    void job(String name, LinkedHashMap args, Closure closure) {
        services[name] = new Job(closure: closure, args: args)
        atLeastOneJobIsLoaded = true
    }

    void runJobs(String... jobNames) {
        jobNames.each {jobName ->
            runJob jobName
        }
    }

    void runJob(String jobName) {

        try {

            Assert.isTrue(isAtLeastOneJobIsLoaded(), "no jobs have been loaded!")
            log.info "beginning job ${jobName}"
            Assert.isTrue services.variables.containsKey(jobName), "the job ${jobName} cannot be run since it has not been loaded"

            def job = services[jobName]
            if (job.args) {
                String dependencyJobName = job.args.dependsOn
                if (job.args.dependsOn) {
                    runJob dependencyJobName
                }
            }

            Assert.isTrue job instanceof Job, "service '${jobName}' cannot be run as a '${Job.class.name}' since it" +
                    "is type '${job.class.name}'"

            def Closure closure = job.closure
            def jobRunner = new JobRunner(jobName: jobName, services: services)
            jobRunner.run closure
        } catch (Exception ex) {
            log.error("Could not complete job ${jobName} due to 1 or more errors")
            throw ex
        }
    }

    def methodMissing(String name, args) {
        if (firstPassComplete) {
            throw new IllegalArgumentException("method ${name} with args ${args} is not allowed in the job dsl")
        }
        Assert.notEmpty(args, "method ${name} is not legal in job dsl with args ${args}")


        if (!isJob(args)) {
            throw new MissingMethodException(name, JobManager.class, args)
        }

        if (args.size() == 1) {
            job name, null, args[0]
        } else {
            job name, args[0], args[1]
        }
    }

    def propertyMissing(String name) {
        if (services.variables.containsKey(name)) {
            return services[name]
        }

        throw new MissingPropertyException(name, JobManager.class)
    }

    boolean isAtLeastOneJobIsLoaded() {
        if (atLeastOneJobIsLoaded) {
            return atLeastOneJobIsLoaded
        }

        boolean result
        def variables = services.variables
        variables.values().each {
            if (it instanceof Job) {
                result = true
            }
        }

        return result
    }

    private boolean isJob(args) {
        if (args.size() == 1) {
            return args[0] instanceof Closure
        }
        if (args.size() == 2) {
            return args[0] instanceof LinkedHashMap && args[1] instanceof Closure
        }

        return false
    }
}

