/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package metridoc.dsl

import org.slf4j.LoggerFactory
import metridoc.utils.Assert

/**
 *
 * @author tbarker
 */

class JobRunner extends PropertyMissing {

    def String jobName
    def log = LoggerFactory.getLogger(JobRunner.class)
    def Exception lastException
    def boolean atLeastOneJobIsLoaded = false
    def List<Class> plugins = []

    def run(Closure closure) {
        closure.setDelegate(this)
        closure.setDirective(Closure.DELEGATE_FIRST)
        closure.setResolveStrategy(Closure.DELEGATE_FIRST)

        def String preKey = "${Services.PRE}${jobName}".toLowerCase()
        def String postKey = "${Services.POST}${jobName}".toLowerCase()

        if(services.variables.containsKey(preKey)) {
            Closure preHook = services[preKey]
            preHook(this)
        }

        closure()

        if (lastException) {
            throw lastException
        }

        if(services.variables.containsKey(postKey)) {
            Closure postHook = services[postKey]
            postHook(this)
        }
    }

    def runJobs(String... jobs) {
        jobs.each {job ->
            runJob job
        }
    }

    def runJob(String jobName) {
        try {
            Assert.isTrue(isAtLeastOneJobIsLoaded(), "no jobs have been loaded!")
            log.info "beginning job ${jobName}"
            Assert.isTrue services.variables.containsKey(jobName), "the job ${jobName} cannot be run since it has not been loaded"

            def job = services[jobName]
            if (job.args) {
                String dependencyJobName = job.args.dependsOn
                if (job.args.dependsOn) {
                    runJob dependencyJobName
                }
            }

            Assert.isTrue job instanceof Job, "service '${jobName}' cannot be run as a '${Job.class.name}' since it" +
                    "is type '${job.class.name}'"

            def Closure closure = job.closure
            def jobRunner = new JobRunner(jobName: jobName, services: services)
            jobRunner.run closure
        } catch (Exception ex) {
            log.error("Could not complete job ${jobName} due to 1 or more errors")
            throw ex
        }
    }

    boolean isAtLeastOneJobIsLoaded() {
        if (atLeastOneJobIsLoaded) {
            return atLeastOneJobIsLoaded
        }

        boolean result
        def variables = services.variables
        variables.values().each {
            if (Job.isInstance(it)) {
                result = true
            }
        }

        atLeastOneJobIsLoaded = result
    }

    /**
     * runs the default job
     * @return return value of the default job
     */
    def runJob() {
        runJob(JobBuilder.DEFAULT_JOB)
    }

    def runJob(Class<Script> scriptClass) {
        def script = scriptClass.newInstance()
        runJob(script)
    }

    def runScript(Class<Script> scriptClass, boolean propagateBinding = true) {
        def script = scriptClass.newInstance()
        runScript(script)
    }

    def runJob(Script script) {

        script.binding = services
        script.metaClass.mixin JobRunner
        JobRunner jobRunner = script as JobRunner
        jobRunner.setServices(services)
        jobRunner.setPlugins(plugins)
        plugins.each {
            script.metaClass.mixin it
        }

        return script.run()
    }

    def runScript(Script script, boolean propagateBinding = true) {
        if(propagateBinding) {
            script.binding = this.services
        }

        script.run()
    }

    def jobs(Closure closure) {
        new JobsHelper(closure: closure, services: services).run()
    }
}




